//
//  ViewBanners.swift
//  Flip
//
//  Created by Anatoliy Matveev on 14.11.2017.
//  Copyright © 2017 Flip. All rights reserved.
//


import Foundation
import UIKit
import FlexLayout
import AVFoundation


class ViewBanners: UIView {
    
    var title: String? {
        didSet {
            if let text = title {
                titleButton.setTitle(text, for: .normal)
                titleButton.flex.markDirty()
                titleButton.flex.isIncludedInLayout(true)
                layout()
            }
        }
    }
    
    
    
    var url: String?
    
    weak var transitionURL: TransitionURL? {
        didSet {
            collectionBanners.transitionURL = transitionURL
        }
    }
    
    weak var pageImageControll: PageImageControll? {
        didSet {
            collectionBanners.pageImageControll = pageImageControll
        }
    }
    
//    weak var sectionPush: SectionPush? {
//        didSet {
//            collectionSections.sectionPush = sectionPush
//        }
//    }
    
    var heightCollection: CGFloat = 0
    
    
    lazy var titleButton: UIButton = {
        let button = UIButton(type: .system)
        button.titleLabel?.font = fonTitleLabel
        button.contentHorizontalAlignment = .left
        button.titleLabel?.adjustsFontSizeToFitWidth = true
        button.titleLabel?.numberOfLines = 2
        button.setTitleColor(colorLabel, for: .normal)
        button.addTarget(self, action: #selector(handleTitleButton), for: .touchUpInside)
        return button
    }()
    
    
    let collectionBanners: CollectionBanners = {
        let layout = UICollectionViewFlowLayout()
        layout.scrollDirection = .horizontal
        let collectionView = CollectionBanners(frame: .zero, collectionViewLayout: layout)
        collectionView.showsHorizontalScrollIndicator = false
        collectionView.isPagingEnabled = true
        return collectionView
    }()
    
    
    
    let rootFlexLayout = UIView()
    
    
    override func layoutSubviews() {
        super.layoutSubviews()
        
        rootFlexLayout.frame.size.width = self.frame.width
        
        layout()
    }
    
    
    
    override init(frame: CGRect) {
        super.init(frame: frame)
        
        self.backgroundColor = backGround
        
        
        reset()
        
        rootFlexLayout.flex.define { (flex) in
            flex.addItem(titleButton).height(40).marginHorizontal(16)
            flex.addItem(collectionBanners)
        }
        
        
        addSubview(rootFlexLayout)
        
    }
    

    func calculateSizeBanners() {
        
        let size = CGSize(width: 1200, height: 500)
        let boundingRect = CGRect(x: 0, y: 0, width: frame.width - 8, height: CGFloat.greatestFiniteMagnitude)
        let rect = AVMakeRect(aspectRatio: size, insideRect: boundingRect)
        
        collectionBanners.flex.height(rect.height)
        collectionBanners.flex.markDirty()
        collectionBanners.reloadData()
    }
    
    
   
    
    init(width: CGFloat) {
        self.init()
        self.frame.size.width = width
        
    }
    
    
    @objc func handleTitleButton(sender: UIButton) {
        let parsUrl = ParsURL()
        print("url headerButton", url as Any)
        parsUrl.urlComponents(url: url) { [weak self] (search) in
            self?.transitionURL?.transitionURL(object: search, title: self?.title)
        }
    }
    
    
    
    
    func layout() {
        calculateSizeBanners()
        rootFlexLayout.flex.layout(mode: .adjustHeight)
    }
    
    
    func reset() {
        titleButton.flex.isIncludedInLayout(false)
    }
    
    
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
}
