//
//  ViewSection.swift
//  Flip
//
//  Created by Anatoliy Matveev on 13.11.2017.
//  Copyright © 2017 Flip. All rights reserved.
//

import Foundation
import UIKit
import FlexLayout


class ViewSection: UIView {
    
    var title: String? {
        didSet {
            if let text = title {
                titleButton.setTitle(text, for: .normal)
                titleButton.flex.markDirty()
                titleButton.flex.isIncludedInLayout(true)
                layout()
            }
        }
    }
    
    
    
    var url: String?
    
    weak var transitionURL: TransitionURL?
    
    weak var sectionPush: SectionPush? {
        didSet {
            collectionSections.sectionPush = sectionPush
        }
    }
    
    var heightCollection: CGFloat = 0
    
    
    lazy var titleButton: UIButton = {
        let button = UIButton(type: .system)
        button.titleLabel?.font = fonTitleLabel
        button.contentHorizontalAlignment = .left
        button.titleLabel?.adjustsFontSizeToFitWidth = true
        button.titleLabel?.numberOfLines = 2
        button.setTitleColor(colorLabel, for: .normal)
        button.addTarget(self, action: #selector(handleTitleButton), for: .touchUpInside)
        return button
    }()
    
    
    let collectionSections: CollectionSections = {
        let layout = UICollectionViewFlowLayout()
        layout.scrollDirection = .vertical
        let collectionView = CollectionSections(frame: .zero, collectionViewLayout: layout)
        return collectionView
    }()
    
    let rootFlexLayout = UIView()
    
    
    override func layoutSubviews() {
        super.layoutSubviews()
        
        rootFlexLayout.frame.size.width = self.frame.width
        
        layout()
    }
    
    
    
    override init(frame: CGRect) {
        super.init(frame: frame)
        
        self.backgroundColor = backGround
        
        self.collectionSections.addObserver(self, forKeyPath: "contentSize", options: NSKeyValueObservingOptions.old, context: nil)
        
        reset()
        
        rootFlexLayout.flex.define { (flex) in
            flex.addItem(titleButton).height(40).marginHorizontal(16)
            flex.addItem(collectionSections)
        }
        
        addSubview(rootFlexLayout)
        
    }
    
    override func observeValue(forKeyPath keyPath: String?, of object: Any?, change: [NSKeyValueChangeKey : Any]?, context: UnsafeMutableRawPointer?) {

        let collectionHeight = self.collectionSections.collectionViewLayout.collectionViewContentSize.height
        calculateCollection(height: collectionHeight)

    }


    func calculateCollection(height: CGFloat) {

        if height != 0 && heightCollection != height {
            heightCollection = height
            collectionSections.flex.height(heightCollection)
            collectionSections.flex.markDirty()
            layout()
        }

    }
    
    deinit {
        collectionSections.removeObserver(self, forKeyPath: "contentSize")
    }
    
    init(width: CGFloat) {
        self.init()
        self.frame.size.width = width
    }
    
    
    @objc func handleTitleButton(sender: UIButton) {
        let parsUrl = ParsURL()
        print("url headerButton", url as Any)
        parsUrl.urlComponents(url: url) { [weak self] (search) in
            self?.transitionURL?.transitionURL(object: search, title: self?.title)
        }
    }
    
    
    
    
    func layout() {
        rootFlexLayout.flex.layout(mode: .adjustHeight)
    }
    
    
    func reset() {
        titleButton.flex.isIncludedInLayout(false)
    }
    
    
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
}
