//
//  CellProductSecV4TV.swift
//  Flip
//
//  Created by Anatoliy Matveev on 15.11.2017.
//  Copyright © 2017 Flip. All rights reserved.
//

import Foundation
import UIKit
import Kingfisher


class CellProductSecV4TV: BaseCellTV {
    
    
    var products: Products? {
        didSet {
            if let imageUrl = products?.imageMedium?.url {
                let url = URL(string: imageUrl)
                imageProduct.kf.indicatorType = .activity
                imageProduct.kf.setImage(with: url, options: [.transition(ImageTransition.fade(0.2))])
                //                imageProduct.loadImage(urlString: imageUrl)
            }
            
            if let flagMarkers = products?.flagMarkers {
                setFlag(flagMarkers: flagMarkers)
            }
            
            if let nameExtra = products?.nameExtra {
                
                var sizeName: CGFloat = 14
                var sizeValue: CGFloat = 11
                
                if UIDevice.current.model == "iPad" {
                    sizeName = 17
                    sizeValue = 14
                }
                
                for (index, value) in nameExtra.enumerated() {
                    switch index {
                    case 0:
                        shortName.attributedText = NSAttributedString(string: value, attributes: [NSAttributedStringKey.font: UIFont.systemFont(ofSize: sizeName), NSAttributedStringKey.foregroundColor: color1])
                    case 1:
                        valueFirst.attributedText = NSAttributedString(string: value, attributes: [NSAttributedStringKey.font: UIFont.systemFont(ofSize: sizeValue), NSAttributedStringKey.foregroundColor: UIColor.gray])
                    case 2:
                        valueSecond.attributedText = NSAttributedString(string: value, attributes: [NSAttributedStringKey.font: UIFont.systemFont(ofSize: sizeValue), NSAttributedStringKey.foregroundColor: UIColor.gray])
                    default:
                        break
                    }
                }
                
            }
            
            if let date = products?.showActivationDate {
                showActivationDate.text = "Начало продаж: " + date
            }
            
            if let price = products?.price, price != "0 тг" {
                
                if let discount = products?.discount, discount != 0 {
                    
                    self.priceLabel.attributedText = NSAttributedString(string: price, attributes: [NSAttributedStringKey.foregroundColor: UIColor.lightGray, NSAttributedStringKey.font: UIFont.systemFont(ofSize: 13), NSAttributedStringKey.strikethroughStyle: 1])
                    
                    self.priceDiscount.text = products?.priceDiscount
                    
                } else {
                    priceLabel.text = price
                }
                
                if let dayLabel = products?.deliveryDay {
                    
                    let attachment = NSTextAttachment()
                    let image = UIImage(named: "clock")?.tint(with: underTheOrder)
                    attachment.image = image
                    attachment.bounds = CGRect(x: 0, y: -2, width: 14, height: 14)
                    let attachmentStr = NSAttributedString(attachment: attachment)
                    let myString = NSMutableAttributedString(string: "")
                    myString.append(attachmentStr)
                    let myString1 = NSMutableAttributedString(string: " " + dayLabel)
                    myString.append(myString1)
                    deliveryDayLabel.attributedText = myString
                    
                }
                
            }
            
            if let idProd = products?.idProduce {
                idProduce = idProd
            }
            
            
            
            if products?.available == Available.notSale {
                notProduct.text = "Товара нет в наличии"
                priceLabel.text = nil
            }
            
        }
    }
    
    var idProduce: Int?
    
    let scale: CGFloat = 0.42
    
    var heightImageProduct: CGFloat {
        return self.contentView.frame.height - 8
    }
    
    let imageProduct: CachedImageView = {
        let imageView = CachedImageView()
        imageView.contentMode = .scaleAspectFit
        imageView.isUserInteractionEnabled = false
        imageView.translatesAutoresizingMaskIntoConstraints = false
        return imageView
    }()
    
    
    //---name---
    let shortName: UILabel = {
        let label = UILabel()
        label.numberOfLines = 4
        label.translatesAutoresizingMaskIntoConstraints = false
        return label
    }()
    
    let valueFirst: UILabel = {
        let label = UILabel()
        label.numberOfLines = 1
        label.translatesAutoresizingMaskIntoConstraints = false
        return label
    }()
    
    let valueSecond: UILabel = {
        let label = UILabel()
        label.numberOfLines = 1
        label.translatesAutoresizingMaskIntoConstraints = false
        return label
    }()
    //---name---
    
    
    let priceLabel: UILabel = {
        let label = UILabel()
        label.numberOfLines = 1
        label.textColor = cololr2//UIColor(r: 31, g: 130, b: 218)
        //        label.adjustsFontSizeToFitWidth = true
        label.textAlignment = .left
        label.font = UIFont.systemFont(ofSize: 13)
        label.translatesAutoresizingMaskIntoConstraints = false
        return label
    }()
    
    let priceDiscount: UILabel = {
        let label = UILabel()
        label.numberOfLines = 1
        label.textColor = cololr2//UIColor(r: 31, g: 130, b: 218)
        //        label.adjustsFontSizeToFitWidth = true
        label.textAlignment = .left
        label.font = UIFont.systemFont(ofSize: 13)
        label.translatesAutoresizingMaskIntoConstraints = false
        return label
    }()
    
    lazy var stackPrice: UIStackView = {
        let stack = UIStackView(arrangedSubviews: [priceLabel, priceDiscount])
        stack.axis = .vertical
        stack.distribution = .fill
        stack.translatesAutoresizingMaskIntoConstraints = false
        return stack
    }()
    
    
    let icondeliveryDay: CachedImageView = {
        let imageView = CachedImageView()
        imageView.image = UIImage(named: "clock")?.withRenderingMode(.alwaysTemplate)
        imageView.tintColor = underTheOrder
        imageView.contentMode = .scaleAspectFit
        imageView.isUserInteractionEnabled = false
        imageView.translatesAutoresizingMaskIntoConstraints = false
        return imageView
    }()
    
    let deliveryDayLabel: UILabel = {
        let label = UILabel()
        label.numberOfLines = 1
        label.textAlignment = .right
        label.adjustsFontSizeToFitWidth = true
        label.textColor = underTheOrder//UIColor(r: 124, g: 124, b: 124)
        label.font = UIFont.systemFont(ofSize: 13)
        label.translatesAutoresizingMaskIntoConstraints = false
        return label
    }()
    
    
    let showActivationDate: UILabel = {
        let label = UILabel()
        label.numberOfLines = 1
        label.adjustsFontSizeToFitWidth = true
        label.textColor = UIColor(r: 124, g: 124, b: 124)
        label.font = UIFont.systemFont(ofSize: 13)
        label.textAlignment = .center
        label.translatesAutoresizingMaskIntoConstraints = false
        return label
    }()
    
    
    //Данного товара сейчас нет в наличии
    //Товара нет в наличии
    
    let notProduct: UILabel = {
        let label = UILabel()
        //        label.text = "Товара нет в наличии"
        label.numberOfLines = 1
        label.textAlignment = .center
        label.textColor = UIColor(r: 124, g: 124, b: 124)
        label.font = UIFont.systemFont(ofSize: 13)
        label.translatesAutoresizingMaskIntoConstraints = false
        return label
    }()
    
    
    lazy var stackLabel: UIStackView = {
        let stack = UIStackView(arrangedSubviews: [shortName, valueFirst, valueSecond])
        stack.axis = .vertical
        stack.distribution = .fill
        stack.spacing = 2
        stack.translatesAutoresizingMaskIntoConstraints = false
        return stack
    }()
    
    lazy var stackPriceAndDay: UIStackView = {
        let stack = UIStackView(arrangedSubviews: [stackPrice, deliveryDayLabel])
        stack.axis = .horizontal
        stack.alignment = .bottom
        stack.distribution = .fill
        stack.spacing = 8
        stack.translatesAutoresizingMaskIntoConstraints = false
        return stack
    }()
    
    
    lazy var stackBottom: UIStackView = {
        let stack = UIStackView(arrangedSubviews: [stackPriceAndDay, showActivationDate, notProduct])
        stack.axis = .vertical
        stack.distribution = .fill
        stack.translatesAutoresizingMaskIntoConstraints = false
        return stack
    }()
    
    
    let containerView = UIView()
    
    override func setupViews() {
        super.setupViews()
        
        contentView.backgroundColor = backGround
        containerView.backgroundColor = .white
        containerView.translatesAutoresizingMaskIntoConstraints = false
        
        contentView.addSubview(containerView)
        
        NSLayoutConstraint.activate([
            containerView.leftAnchor.constraint(equalTo: contentView.leftAnchor, constant: 0),
            containerView.rightAnchor.constraint(equalTo: contentView.rightAnchor, constant: 0),
            containerView.topAnchor.constraint(equalTo: contentView.topAnchor, constant: 8),
            containerView.bottomAnchor.constraint(equalTo: contentView.bottomAnchor, constant: -8),
            ])
        
        containerView.addSubview(imageProduct)
        containerView.addSubview(stackLabel)
        containerView.addSubview(stackBottom)
        
        imageProduct.anchor(top: containerView.topAnchor, left: containerView.leftAnchor, bottom: containerView.bottomAnchor, right: nil, topConstant: 4, leftConstant: 4, bottomConstant: 4, rightConstant: 0, widthConstant: heightImageProduct, heightConstant: 0)
        
        stackLabel.anchor(top: containerView.topAnchor, left: imageProduct.rightAnchor, bottom: nil, right: containerView.rightAnchor, topConstant: 8, leftConstant: 8, bottomConstant: 0, rightConstant: 8, widthConstant: 0, heightConstant: 0)
        
        stackPrice.setContentCompressionResistancePriority(UILayoutPriority(rawValue: 1000), for: .horizontal)
        deliveryDayLabel.setContentCompressionResistancePriority(UILayoutPriority(rawValue: 249), for: .horizontal)
        
        
        stackBottom.anchor(top: nil, left: imageProduct.rightAnchor, bottom: containerView.bottomAnchor, right: containerView.rightAnchor, topConstant: 0, leftConstant: 8, bottomConstant: 8, rightConstant: 8, widthConstant: 0, heightConstant: 0)
        
    }
    
    
    
    
    
    func setFlag(flagMarkers: [FlagMarkers]) {
        
        flagMarkers.forEach {
            guard let position = $0.position else { return }
            guard let shift = $0.shift else { return }
            guard let color = $0.color else { return }
            let flagMark = FlagMark(frame: .zero)
            flagMark.reset()
            flagMark.set(text: $0.text, position: position)
            flagMark.setColor(ColorString(rawValue: color))
            flagMark.sizeToFit()
            addSubview(flagMark)
            
            switch (position, shift) {
            case (PositionFlag.horizontal, Shift.low):
                
                flagMark.frame.origin = CGPoint(x: 8, y: 4)
                
            case (PositionFlag.horizontal, Shift.medium):
                
                flagMark.frame.origin = CGPoint(x: 8, y: heightImageProduct - 16 - flagMark.frame.size.height - 4)
                
            case (PositionFlag.horizontal, Shift.high):
                
                flagMark.frame.origin = CGPoint(x: 8, y: heightImageProduct - 16)
                
            case (PositionFlag.vertical, Shift.low):
                
                flagMark.frame.origin = CGPoint(x: 8, y: 4)
                
            case (PositionFlag.vertical, Shift.medium):
                
                flagMark.frame.origin = CGPoint(x: heightImageProduct - (flagMark.frame.width * 2) - 4, y: 4)
                
            case (PositionFlag.vertical, Shift.high):
                
                flagMark.frame.origin = CGPoint(x: heightImageProduct - flagMark.frame.width, y: 4)
                
            default:
                break
            }
            
        }
    }
    
    
    
    func resetValueLabel() {
        
        self.subviews.forEach {
            if $0 is FlagMark {
                ($0 as! FlagMark).removeFromSuperview()
            }
        }
        
        imageProduct.image = nil
        shortName.text = nil
        shortName.attributedText = nil
        notProduct.text = nil
        valueFirst.text = nil
        valueSecond.text = nil
        priceLabel.text = nil
        priceDiscount.text = nil
        deliveryDayLabel.text = nil
        showActivationDate.text = nil
    }
    
    
}
