//
//  ViewBanners.swift
//  Flip
//
//  Created by Anatoliy Matveev on 13.11.2017.
//  Copyright © 2017 Flip. All rights reserved.
//

import Foundation
import UIKit
import FlexLayout

class ViewProduct: UIView {
    
    
    var title: String? {
        didSet {
            if let text = title {
                titleButton.setTitle(text, for: .normal)
                titleButton.flex.markDirty()
                titleButton.flex.isIncludedInLayout(true)
                layout()
            }
        }
    }
    
    
    
    var url: String?
    
    weak var transitionURL: TransitionURL?
    
    weak var productSectionPush: ProductSectionPush? {
        didSet {
            productSection.productSectionPush = productSectionPush
        }
    }
    
    lazy var titleButton: UIButton = {
        let button = UIButton(type: .system)
        button.titleLabel?.font = fonTitleLabel
        button.contentHorizontalAlignment = .left
        button.titleLabel?.adjustsFontSizeToFitWidth = true
        button.titleLabel?.numberOfLines = 2
        button.setTitleColor(colorLabel, for: .normal)
        button.addTarget(self, action: #selector(handleTitleButton), for: .touchUpInside)
        return button
    }()
    
    
    let productSection: ProductSection = {
        let layout = UICollectionViewFlowLayout()
        layout.scrollDirection = .horizontal
        let collectionView = ProductSection(frame: .zero, collectionViewLayout: layout)
        return collectionView
    }()
    
    let rootFlexLayout = UIView()
    
    

    override init(frame: CGRect) {
        super.init(frame: frame)
        
        self.backgroundColor = backGround
        
//        titleButton.translatesAutoresizingMaskIntoConstraints = false
//        productSection.translatesAutoresizingMaskIntoConstraints = false
//
//        self.addSubview(titleButton)
//        self.addSubview(productSection)
//
//
//        NSLayoutConstraint.activate([
//            titleButton.leftAnchor.constraint(equalTo: leftAnchor, constant: 16),
//            titleButton.rightAnchor.constraint(equalTo: rightAnchor, constant: -16),
//            titleButton.topAnchor.constraint(equalTo: topAnchor, constant: 0),
//            titleButton.heightAnchor.constraint(lessThanOrEqualToConstant: 40),
//
//            productSection.leftAnchor.constraint(equalTo: leftAnchor),
//            productSection.rightAnchor.constraint(equalTo: rightAnchor),
//            productSection.topAnchor.constraint(equalTo: titleButton.bottomAnchor),
//            productSection.bottomAnchor.constraint(equalTo: bottomAnchor),
//            productSection.heightAnchor.constraint(lessThanOrEqualToConstant: 328)
//
//            ])
        
        reset()

        rootFlexLayout.flex.define { (flex) in
            flex.addItem(titleButton).height(40).marginHorizontal(16)
            flex.addItem(productSection).height(328)
        }
        
        addSubview(rootFlexLayout)
    }
    
    init(width: CGFloat) {
        self.init()
        self.frame.size.width = width
    }
    
    
    @objc func handleTitleButton(sender: UIButton) {
        let parsUrl = ParsURL()
        print("url headerButton", url as Any)
        parsUrl.urlComponents(url: url) { [weak self] (search) in
            self?.transitionURL?.transitionURL(object: search, title: self?.title)
        }
    }
    
    
    
    override func layoutSubviews() {
        super.layoutSubviews()
        
        rootFlexLayout.frame.size.width = self.frame.width
        
        layout()
    }
    
    fileprivate func layout() {
        rootFlexLayout.flex.layout(mode: .adjustHeight)
    }
    
    
    func reset() {
        titleButton.flex.isIncludedInLayout(false)
    }
    
    
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    
}

