//
//  ScrollCatalog.swift
//  Flip
//
//  Created by Anatoliy Matveev on 10.11.2017.
//  Copyright © 2017 Flip. All rights reserved.
//

import Foundation
import UIKit
import FlexLayout
import PinLayout


class ScrollCatalog: UIViewController, UIScrollViewDelegate {
    
    
    var modelView: ModelView?
    var indexTag = 0
    
    var heightProdSecV4: CGFloat {
        if UIDevice.current.model == "iPad" {
            return 250
        } else {
            return 150
        }
    }
    
    
    var linkView = [[String: UIView]]()
    
    var loadFilterFon: LoadFilterFon?
    var refreshControl = UIRefreshControl()
    var refreshCollection = UIRefreshControl()
    
    var categories = false
    var urlSearch = false
    var wordSearch = false
    var descript = false
    var searchPublish = false
    var discount = false
    
    var filters = [[String: Any]]()
    
    //--filterValue
    var search = ApiKey.productSearch
    
    var nameSort: String = "Сортировать"
    
    var indexFilter: Int?
    var indexSection: Int?
    
    
    let fonView: UIView = {
        let view = UIView()
        view.backgroundColor = backGround
        view.translatesAutoresizingMaskIntoConstraints = false
        return view
    }()
    
    let spinner: UIActivityIndicatorView = {
        let spinner = UIActivityIndicatorView(activityIndicatorStyle: UIActivityIndicatorViewStyle.whiteLarge)
        spinner.color = .gray
        spinner.translatesAutoresizingMaskIntoConstraints = false
        return spinner
    }()
    
    
    
    //-- filter
    let menuView: UIView = {
        let view = UIView()
        view.backgroundColor = .white//UIColor(white: 0.95, alpha: 1)
//        view.translatesAutoresizingMaskIntoConstraints = false
        return view
    }()
    
    lazy var filterButton: CustomButton = {
        let button = CustomButton(type: .system)
        //        button.setTitle("Фильтр", for: .normal)
        button.addTarget(self, action: #selector(editFilterMenu), for: .touchUpInside)
        return button
    }()
    
    lazy var sortButton: CustomButton = {
        let button = CustomButton(type: .system)
        button.contentHorizontalAlignment = .left
        button.setTitle(self.nameSort, for: .normal)
        button.addTarget(self, action: #selector(editSortMenu), for: .touchUpInside)
        return button
    }()
    //--endfilter
    
    
    
    
    // search
    lazy var textFiledSearch: TextFieldSearch = {
        let tx = TextFieldSearch()
        tx.keyboardType = .default
        tx.returnKeyType = .search
        tx.delegate = self
        tx.tintColor = color3
        tx.spellCheckingType = .no
        tx.autocorrectionType = .no
        return tx
    }()
    
    var firstAnimation = true
    
    var setTitle: String!
    
    var keywords: [String]? {
        didSet {
            tableView.reloadSections([0], with: .fade)
        }
    }
    
    
    lazy var tableView: TableView = {
        let tableView = TableView(frame: .zero, style: .plain)
        tableView.backgroundColor = backGround
        tableView.separatorStyle = .none
        tableView.delegate = self
        tableView.dataSource = self
        return tableView
    }()
    
    lazy var rightBarButton: UIButton = {
        let button = UIButton(type: .system)
        button.setTitle("Отменить", for: .normal)
        button.titleLabel?.font = UIFont.systemFont(ofSize: 18)
        button.setTitleColor(.white, for: .normal)
        button.contentHorizontalAlignment = .right
        button.addTarget(self, action: #selector(cancelledButton), for: .touchUpInside)
        return button
    }()
    
    let contetnView: UIView = {
        let view = UIView()
        return view
    }()
    // search
    
    
    
    
    
    
    
    
    let pageControl: UIPageControl = {
        let page = UIPageControl()
        page.currentPage = 0
        page.pageIndicatorTintColor = UIColor.gray
        page.currentPageIndicatorTintColor = UIColor.red
        return page
    }()
    
    
    let menufilter = UINavigationController(rootViewController: FilterMenu())
    let menuSort = SortMenu()
    var isSetupFilter = true
    
    
    
    
    lazy var blackViewSortMenu: UIView = {
        let view = UIView()
        view.addGestureRecognizer(UITapGestureRecognizer(target: self, action: #selector(editSortMenu)))
        view.backgroundColor = UIColor(white: 0, alpha: 0.5)
        view.translatesAutoresizingMaskIntoConstraints = false
        return view
    }()
    
    lazy var blackView: UIView = {
        let view = UIView()
        view.addGestureRecognizer(UITapGestureRecognizer(target: self, action: #selector(handleBlackView)))
        view.addGestureRecognizer(UISwipeGestureRecognizer(target: self, action: #selector(handleSwipeCloseMenu)))
        view.backgroundColor = UIColor(white: 0, alpha: 0.5)
        view.translatesAutoresizingMaskIntoConstraints = false
        return view
    }()
    
    
    var heightMenuSort: NSLayoutConstraint?
    
    
    var widthMenu: CGFloat {
        get {
            if UIDevice.current.model == "iPad" {
                return 350
            }
            
            return UIScreen.main.bounds.width * 0.8
        }
        
        set {
            self.widthMenu = newValue
        }
    }
    
    
    

    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    lazy var scrollView: ScrollView = {
        let scroll = ScrollView(frame: .zero)
        scroll.delegate = self
        scroll.backgroundColor = backGround
        return scroll
    }()
    
    
    let rootFlexContainer = UIView()
    
    
    
    
    let collectionBanners: CollectionBanners = {
        let layout = UICollectionViewFlowLayout()
        layout.scrollDirection = .horizontal
        let collectionView = CollectionBanners(frame: .zero, collectionViewLayout: layout)
        collectionView.showsHorizontalScrollIndicator = false
        collectionView.isPagingEnabled = true
        return collectionView
    }()
    
    let catalogMenu: CatalogMenu = {
        let catalog = CatalogMenu(frame: .zero)
        return catalog
    }()
    
    let collectionSections: CollectionSections = {
        let layout = UICollectionViewFlowLayout()
        layout.scrollDirection = .vertical
        let collectionView = CollectionSections(frame: .zero, collectionViewLayout: layout)
        return collectionView
    }()
    
//    let containerBanners = UIView()
//    let containerMenu = UIView()
//    let containerSection = UIView()
//    let containerProduct = UIView()
//    let containerPublisher = UIView()
    
    
    let collectionPublishers: CollectionPublishers = {
        let layout = UICollectionViewFlowLayout()
        layout.scrollDirection = .vertical
        let collectionView = CollectionPublishers(frame: .zero, collectionViewLayout: layout)
        return collectionView
    }()
    
    let prodSecFill: ProdSecFill = {
        let layout = UICollectionViewFlowLayout()
        layout.scrollDirection = .vertical
        let collectionView = ProdSecFill(frame: .zero, collectionViewLayout: layout)
        collectionView.contentInset = UIEdgeInsets(top: 40, left: 0, bottom: 0, right: 0)
        collectionView.scrollIndicatorInsets = UIEdgeInsets(top: 40, left: 0, bottom: 0, right: 0)
        collectionView.isHidden = true
        return collectionView
    }()
    
    
    var json: JSON? {
        didSet {
            
            guard json != nil else { return }
            
            if refreshControl.isRefreshing {
                refresh(json: json!)
                endRefresh()
            } else {
                pars(json: json!)
            }
            
            
            
//            pars(json: json!)
            
            if navigationItem.title == nil {
                navigationItem.title = json?.title
            }
            
            animateLoad()
        }
    }
    
    
    var id: Int? {
        didSet {
            guard id != nil else { return }
            
            
            if let from = search["from"] as? String {
                search["from"] = from
            } else {
                search["from"] = ApiKey.From.catalog
            }
            
            search["limit"] = 20
            search["idSection"] = id
            search["offset"] = 0
            
            if discount {
                let filterI104 = ApiKey.filter(id: FilterID.i104)
                filters.append(filterI104)
                
                search["filter"] = filters
            }
            
        }
    }
    
    
    
    var productSearch: ProductSearch? {
        didSet {
            
            guard productSearch != nil else { return }
            guard let product = productSearch?.products else { return }
            
            parsSearch(products: product)
            
            if isSetupFilter {
                setupFilter()
                isSetupFilter = false
            }

            filterLoadRemove()
            
            if let count = productSearch?.filtersSelected?.count, count != 0 {
                filterButton.setTitle("Фильтр (\(count))", for: .normal)
            } else {
                filterButton.setTitle("Фильтр", for: .normal)
            }
            filterButton.layoutIfNeeded()
            
            if navigationItem.title == nil {
                navigationItem.title = productSearch?.title
            }
            
            
            
            if let menufilter = menufilter.viewControllers.first as? FilterMenu {
                menufilter.idSection = id
                menufilter.productSearch = productSearch
            }
            
            if menufilter.viewControllers.count > 1 {
                if let sectionFilter = menufilter.viewControllers[1] as? SectionFilter {
                    if let filters = productSearch?.filters {
                        
                        for i in filters {
                            if i.id == FilterID.i500 {
                                sectionFilter.filters = i
                                sectionFilter.totalAll = productSearch?.totalAll
                                break
                            }
                        }
                        
                    }
                }
                
                if let vc = menufilter.viewControllers[1] as? ViewController {
                    vc.totalAll = productSearch?.totalAll
                }
                
                if let filterTypeList = menufilter.viewControllers[1] as? FilterTypeList {
                    
                    guard let section = indexSection else { return }
                    filterTypeList.filter = productSearch?.filters?[section]
                    
                }
                
            }
            
            menuSort.orders = productSearch?.orders
            
            if productSearch?.products?.count ?? 0 <= search["limit"] as! Int {
                animateLoad()
            }
            
        }
    }
    
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
//        reset()
        
        fetchSubsection()
        
//        // стираем текст с кнопки вернуться
        navigationItem.backBarButtonItem = UIBarButtonItem(title: "", style: .plain, target: nil, action: nil)
        
        self.navigationController?.navigationBar.setBackgroundImage(nil, for: UIBarMetrics.default)
        self.navigationController?.navigationBar.shadowImage = nil
        
        navigationController?.navigationBar.barTintColor = barTintColor
        
//        refreshControl.addTarget(self, action: #selector(refreshData), for: .valueChanged)
        refreshControl.backgroundColor = backGround
        refreshControl.superview?.backgroundColor = refreshControl.backgroundColor
        
//        refreshCollection.addTarget(self, action: #selector(refreshData), for: .valueChanged)
        refreshCollection.backgroundColor = backGround
        refreshCollection.superview?.backgroundColor = refreshCollection.backgroundColor
        
        scrollView.refreshControl = refreshControl
        prodSecFill.refreshControl = refreshCollection
        
        
        
        rootFlexContainer.flex.define { (flex) in
            
        }
        
        scrollView.addSubview(rootFlexContainer)
        view.addSubview(scrollView)
        
        scrollView.addSubview(pageControl)
        
        view.addSubview(prodSecFill)
        
        
        if wordSearch {
            contetnView.addSubview(textFiledSearch)
            contetnView.addSubview(rightBarButton)
            navigationItem.titleView = contetnView
            
            let tapScreen = UITapGestureRecognizer(target: self, action: #selector(handleTap))
            tapScreen.cancelsTouchesInView = false
            view.addGestureRecognizer(tapScreen)
            view.addSubview(tableView)
            
            tableView.registerCell(SearchCell.self)
            tableView.tableFooterView = UIView(frame: .zero)
        }
        
        load()
    }
    
    
    override func viewWillDisappear(_ animated: Bool) {
        super.viewWillDisappear(animated)
        
        textFiledSearch.resignFirstResponder()
    }

    
    override func viewDidLayoutSubviews() {
        super.viewDidLayoutSubviews()
        
        // 1) Layout the contentView & rootFlexContainer using PinLayout
        scrollView.frame = view.frame
        rootFlexContainer.frame.size.width = view.frame.width
        
        // 2) Let the flexbox container layout itself and adjust the height
        rootFlexContainer.flex.layout(mode: .adjustHeight)
        
//        print("rootFlexContainer.frame.size", rootFlexContainer.frame.size)
        
        // 3) Adjust the scrollview contentSize
        scrollView.contentSize = rootFlexContainer.frame.size
        
        prodSecFill.frame = view.frame
        
        if let viewBanners = modelView?.viewBanners {
            pageControl.frame = CGRect(x: 0, y: viewBanners.frame.height - 25, width: viewBanners.frame.width, height: 25)
        }
        
        
        if wordSearch {
            var width: CGFloat = 16
            
            if UIDevice.current.model == "iPad" {
                width += 8
            }
            
            if let navBarFrame = self.navigationController?.navigationBar.frame {
                contetnView.frame = CGRect(x: 0, y: 0, width: navBarFrame.width - width, height: navBarFrame.height)
                currentSearch(style: textFiledSearch.style)
            }
        }
        
    }
    
//    override func viewWillAppear(_ animated: Bool) {
//        super.viewWillAppear(animated)
//
//        print("willAppear scrollView.contentSize", scrollView.contentSize)
//    }
    
    override func viewWillTransition(to size: CGSize, with coordinator: UIViewControllerTransitionCoordinator) {
        super.viewWillTransition(to: size, with: coordinator)

        print("test test test test test test test")

        if !prodSecFill.isHidden {
            prodSecFill.reloadData()
            prodSecFill.collectionViewLayout.invalidateLayout()
        }

//        layout()

        coordinator.animate(alongsideTransition: { [weak self] (context) in
            
            guard let strongSelf = self else { return }
            
            if strongSelf.wordSearch {
                if !strongSelf.tableView.isHidden {
                    strongSelf.tableView.frame = strongSelf.view.frame
                }
            }
            
            if strongSelf.textFiledSearch.isEditing {
                strongSelf.textFiledSearch.pin.left().width(strongSelf.contetnView.frame.size.width - 94).height(30).vCenter()
                strongSelf.rightBarButton.pin.right(of: strongSelf.textFiledSearch).right().vCenter()
            } else {
                let width = strongSelf.textFiledSearch.text?.sizeOfString(font: strongSelf.textFiledSearch.font!, constrainedToWidth: strongSelf.contetnView.frame.size.width - 94)
                strongSelf.textFiledSearch.pin.hCenter(to:  strongSelf.navigationController!.navigationBar.edge.hCenter).height(30).vCenter().width(width!.width + 74)
                strongSelf.rightBarButton.pin.right().height(44).width(0).vCenter()
            }
            
            strongSelf.layout()
            
        }) { (completed) in

        }


    }
    
//    override func willTransition(to newCollection: UITraitCollection, with coordinator: UIViewControllerTransitionCoordinator) {
//        super.willTransition(to: newCollection, with: coordinator)
//
//
//        coordinator.animate(alongsideTransition: { (contex) in
//            self.layout()
//        }) { (comleted) in
//
//        }
//    }
//
    
    
    
    
    
    func layout() {
        rootFlexContainer.flex.layout(mode: .adjustHeight)
    }
    
    
    func scrollViewDidEndDragging(_ scrollView: UIScrollView, willDecelerate decelerate: Bool) {
        
        refresh()
    }
    
    func scrollViewDidScroll(_ scrollView: UIScrollView) {
        
        let height = scrollView.frame.size.height
        let contentYOffset = scrollView.contentOffset.y
        let distanceFromBottom = scrollView.contentSize.height - contentYOffset
        let distance = distanceFromBottom - height
        
        print("distance", distance)
        print("heightProdSecV4 * 3", heightProdSecV4 * 3)
        
        if distance == (heightProdSecV4 * 3) {
            DispatchQueue.main.async {
                guard let count = self.productSearch?.products?.count else { return }
                guard let totalAll = self.productSearch?.totalAll else { return }
                
                guard Int(totalAll) != count else { return }
                
                self.search["offset"] = count
                self.searchProduct()
            }
        }
    }
    
    
    func endRefresh() {
        DispatchQueue.main.asyncAfter(deadline: .now() + 0.3) {

            self.refreshControl.endRefreshing()
            self.refreshCollection.endRefreshing()
            
            self.prodSecFill.isUserInteractionEnabled = true
            self.scrollView.isUserInteractionEnabled = true
            self.menuView.isUserInteractionEnabled = true
        }
    }
    
    func reset() {
        
        collectionBanners.flex.markDirty()
        catalogMenu.flex.markDirty()
        collectionSections.flex.markDirty()
        collectionPublishers.flex.markDirty()
        prodSecFill.flex.markDirty()
        
        
        collectionBanners.flex.isIncludedInLayout(false)
        catalogMenu.flex.isIncludedInLayout(false)
        collectionSections.flex.isIncludedInLayout(false)
        collectionPublishers.flex.isIncludedInLayout(false)
        prodSecFill.flex.isIncludedInLayout(false)
        
        
        layout()
    }
    
    
    
}


































