//
//  SearchWordsScroll.swift
//  Flip
//
//  Created by Anatoliy Matveev on 15.11.2017.
//  Copyright © 2017 Flip. All rights reserved.
//

import Foundation
import UIKit
import PinLayout


extension ScrollCatalog: UITextFieldDelegate, UITableViewDataSource, UITableViewDelegate {
    
    func textField(_ textField: UITextField, shouldChangeCharactersIn range: NSRange, replacementString string: String) -> Bool {
        
        NSObject.cancelPreviousPerformRequests(withTarget: self, selector: #selector(getTextField), object: textField)
        self.perform(#selector(getTextField), with: textField, afterDelay: 0.4)
        
        return true
    }
    
    
    
    func currentSearch(style: SearchStyle) {
        switch style {
        case .search:
            self.navigationController?.navigationBar.layoutIfNeeded()
            textFiledSearch.frame = CGRect(x: 0, y: 7, width: contetnView.frame.size.width - 94, height: 30)
            rightBarButton.frame = CGRect(x: contetnView.frame.size.width - 94, y: 0, width: 94, height: 44)
        case .title, .titleNone:
            self.navigationController?.navigationBar.layoutIfNeeded()
            let width = textFiledSearch.text?.sizeOfString(font: textFiledSearch.font!, constrainedToWidth: contetnView.frame.size.width - 94)
            textFiledSearch.pin.hCenter(to: self.navigationController!.navigationBar.edge.hCenter).height(30).vCenter().width(width!.width + 74)
            rightBarButton.pin.right().height(44).width(0).vCenter()
        }
        
    }
    
    func textFieldShouldClear(_ textField: UITextField) -> Bool {
        textField.text = ""
        textFiledSearch.text = ""
        return true
    }
    
    func textFieldShouldReturn(_ textField: UITextField) -> Bool {
        
        if textField.returnKeyType == .search {
            if textField.text != "" && textField.text != nil {
                transitionCatalog(word: textField.text)
                textField.resignFirstResponder()
            }
        }
        
        return true
    }
    
    func textFieldShouldBeginEditing(_ textField: UITextField) -> Bool {
        return true
    }
    
    func textFieldDidBeginEditing(_ textField: UITextField) {
        animationShow()
    }
    
    //    func textFieldDidEndEditing(_ textField: UITextField) {
    //        animationCancel()
    //    }
    
    
    @objc func handleTap() {
        //animationCancel()
        textFiledSearch.resignFirstResponder()
    }
    
    @objc func cancelledButton(sender: UIButton) {
        animationCancel()
        textFiledSearch.resignFirstResponder()
    }
    
    func scrollViewWillBeginDragging(_ scrollView: UIScrollView) {
        //        animationCancel()
        textFiledSearch.resignFirstResponder()
    }
    
    
    
    @objc func getTextField(textField: UITextField) {
        
        guard let word = textField.text, !word.isEmpty else {
            return
        }
        
        if !word.trimmingCharacters(in: .whitespaces).isEmpty {
            let handler = Handler(url: ApiKey.getProductKeywords, extra: .query(word))
            handler.getKeywodrs(completion: { [weak self] (result) in
                guard let strongSelf = self else { return }
                switch result {
                case .Success(let keywords):
                    strongSelf.keywords = keywords
                case .Error(let error):
                    DispatchQueue.main.async {
                        ErrorProcessing.shared.alert(error: error)
                    }
                }
            })
        }
        
    }
    
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return keywords?.count ?? 0
    }
    
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        
        let cell = tableView.dequeueCell(for: indexPath) as SearchCell
        cell.backgroundColor = backGround
        cell.textLabel?.text = keywords?[indexPath.row]
        return cell
    }
    
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        return 40
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        let word = keywords?[indexPath.row]
        transitionCatalog(word: word)
    }
    
    
    
    
    
    func transitionCatalog(word: String?) {
        setTitle = word
        animationCancel()
        refreshanimateLoad()
        prodSecFill.products = nil
        prodSecFill.reloadSections([0])
        search["limit"] = 20
        search["from"] = ApiKey.From.search
        search["offset"] = 0
        search["query"] = word
        AppDelegate.changeFilter = false
        searchProduct()
        tableView.isHidden = true
    }
    
    
    func animationCancel() {
        menuView.isHidden = false
        tableView.isHidden = true
        //        textFiledSearch.resignFirstResponder()
        
        if textFiledSearch.style == SearchStyle.search {
            if self.textFiledSearch.text != "" {
                self.textFiledSearch.rightView?.alpha = 0
            } else {
                self.textFiledSearch.rightView?.alpha = 1
            }
            self.contetnView.layoutIfNeeded()
            UIView.animate(withDuration: 0.5, delay: 0, usingSpringWithDamping: 1, initialSpringVelocity: 1, options: .curveEaseInOut, animations: {
                self.textFiledSearch.frame = CGRect(x: 0, y: 7, width: self.contetnView.frame.size.width, height: 30)
                self.rightBarButton.frame = CGRect(x: self.contetnView.frame.size.width, y: 0, width: 0, height: 44)
                self.contetnView.layoutIfNeeded()
            }) { (complete) in
                UIView.animate(withDuration: 0.2, animations: {
                    if self.textFiledSearch.text != "" {
                        self.textFiledSearch.rightView?.alpha = 1
                    }
                })
            }
        } else {
            self.contetnView.layoutIfNeeded()
            UIView.animate(withDuration: 0.5, delay: 0, usingSpringWithDamping: 1, initialSpringVelocity: 1, options: .curveEaseInOut, animations: {
                self.textFiledSearch.leftView?.alpha = 0
                if self.textFiledSearch.text != "" {
                    self.textFiledSearch.rightView?.isHidden = true
                } else {
                    self.textFiledSearch.rightView?.alpha = 0
                }
                
                self.textFiledSearch.text = self.setTitle
                
                self.textFiledSearch.borderStyle = .none
                self.textFiledSearch.textColor = UIColor.white
                self.textFiledSearch.backgroundColor = .clear
                
                let width = self.textFiledSearch.text?.sizeOfString(font: self.textFiledSearch.font!, constrainedToWidth: self.contetnView.frame.size.width - 94)
                self.textFiledSearch.pin.hCenter(to: self.navigationController!.navigationBar.edge.hCenter).height(30).vCenter().width(width!.width + 74)
                self.rightBarButton.pin.right().height(44).width(0).vCenter()
                self.contetnView.layoutIfNeeded()
            }) { (complete) in
                
            }
        }
        
    }
    
    func animationShow() {
        
        tableView.frame = view.frame
        tableView.isHidden = false
        menuView.isHidden = true
        
        if textFiledSearch.style == SearchStyle.search {
            
            if firstAnimation {
                textFiledSearch.frame = CGRect(x: 0, y: 7, width: contetnView.frame.size.width - 94, height: 30)
                rightBarButton.frame = CGRect(x: contetnView.frame.size.width, y: 0, width: 0, height: 44)
                firstAnimation = false
            }
            
            self.contetnView.layoutIfNeeded()
            UIView.animate(withDuration: 0.5, delay: 0, usingSpringWithDamping: 1, initialSpringVelocity: 1, options: .curveEaseInOut, animations: {
                self.textFiledSearch.frame = CGRect(x: 0, y: 7, width: self.contetnView.frame.size.width - 94, height: 30)
                self.rightBarButton.frame = CGRect(x: self.contetnView.frame.size.width - 94, y: 0, width: 94, height: 44)
                self.contetnView.layoutIfNeeded()
            }) { (complete) in
                
            }
            
        } else {
            self.contetnView.layoutIfNeeded()
            UIView.animate(withDuration: 0.5, delay: 0, usingSpringWithDamping: 1, initialSpringVelocity: 1, options: .curveEaseInOut, animations: {
                
                self.textFiledSearch.borderStyle = .roundedRect
                self.textFiledSearch.textColor = UIColor.black
                self.textFiledSearch.backgroundColor = backGround
                self.textFiledSearch.rightView?.isHidden = false
                self.textFiledSearch.rightView?.alpha = 1
                self.textFiledSearch.leftView?.alpha = 1
                self.textFiledSearch.pin.left().width(self.contetnView.frame.size.width - 94).height(30).vCenter()
                self.rightBarButton.pin.right(of: self.textFiledSearch).right().vCenter()
                self.contetnView.layoutIfNeeded()
            }) { (complete) in
                
            }
        }
    }
    
    
    func getKeywodrs(word: String?) {
        GetProductKeywords.sharedInstance.getDataWith(query: word) { [weak self] (result) in
            guard let strongSelf = self else { return }
            switch result {
            case .Success(let keywords):
                strongSelf.keywords = keywords
            case .Error(let error):
                DispatchQueue.main.async {
                    ErrorProcessing.shared.alert(error: error)
                }
            }
            
        }
    }
    
}
