//
//  parsScrollCatalog.swift
//  Flip
//
//  Created by Anatoliy Matveev on 10.11.2017.
//  Copyright © 2017 Flip. All rights reserved.
//

import Foundation
import UIKit
import FlexLayout

extension ScrollCatalog {
    

    func pars(json: JSON) {
        
        var index = 0
        
        self.modelView = ModelView()
        
        json.blocks?.forEach {
            
            switch $0.type ?? "" {

            case Type.banners:
                
                guard let banners = $0.banners else { return }
                guard banners.count != 0 else { return }
                
                
                let viewBanners = ViewBanners(width: self.view.frame.size.width)
                self.modelView?.viewBanners = viewBanners
                
                viewBanners.transitionURL = self
                viewBanners.pageImageControll = self
                viewBanners.title = nil
                viewBanners.url = $0.url
                viewBanners.collectionBanners.get(banners: banners, reload: true)
                viewBanners.flex.markDirty()
                rootFlexContainer.flex.addItem(viewBanners).shrink(1).grow(1)
                

//                collectionBanners.banners = banners
//                collectionBanners.transitionURL = self
//                calculateSizeBanners()
//                rootFlexContainer.flex.addItem(collectionBanners)

            case Type.menu:
                
//                print("catalogMenu")

                catalogMenu.allCategoriesPush = self
                rootFlexContainer.flex.addItem(catalogMenu).height(40).marginVertical(8)
                catalogMenu.flex.isIncludedInLayout(true)
                catalogMenu.flex.markDirty()

            case Type.sections:
                
//                print("viewSection")
                
                guard let sections = $0.sections else { return }
                guard sections.count != 0 else { return }
                
                let viewSection = ViewSection(width: self.view.frame.size.width)
                self.modelView?.viewSection = viewSection
                
                viewSection.transitionURL = self
                viewSection.sectionPush = self
                viewSection.title = $0.title
                viewSection.url = $0.url
                viewSection.collectionSections.get(sections: sections, reload: true)
                viewSection.flex.markDirty()
                rootFlexContainer.flex.addItem(viewSection).shrink(1).grow(1)
                
                

            case Type.producesSection, Type.produces:

//                print("viewProduct")
                
                guard let products = $0.products else { return }
                guard products.count != 0 else { return }
                
                let viewProduct = ViewProduct(width: self.view.frame.size.width)
                self.modelView?.viewProducts?["\(index)"] = viewProduct
                
                viewProduct.transitionURL = self
                viewProduct.productSectionPush = self
                viewProduct.title = $0.title
                viewProduct.url = $0.url
                viewProduct.productSection.get(products: products, reload: true)
                viewProduct.flex.markDirty()
                rootFlexContainer.flex.addItem(viewProduct)

                index += 1
                
            case Type.publishers:
                
//                print("viewPublisher")
                
                guard let publishers = $0.publishers else { return }
                guard publishers.count != 0 else { return }
                
                let viewPublisher = ViewPublisher(width: self.view.frame.size.width)
                self.modelView?.viewPublisher = viewPublisher
                
                viewPublisher.transitionURL = self
                viewPublisher.publisherPush = self
                viewPublisher.title = $0.title
                viewPublisher.url = $0.url
                viewPublisher.collectionPublishers.get(publisher: publishers, reload: true)
                viewPublisher.flex.markDirty()
                rootFlexContainer.flex.addItem(viewPublisher).shrink(1).grow(1)
                

            default:
                break
            }
        }
        
        
        layout()

    }
    
    

    func refresh(json: JSON) {
        
        var index = 0
        
        json.blocks?.forEach {
            
            switch $0.type ?? "" {
                
            case Type.banners:
                
                guard let banners = $0.banners else { return }
                guard banners.count != 0 else { return }
                
                
                self.modelView?.viewBanners?.transitionURL = self
                self.modelView?.viewBanners?.pageImageControll = self
                self.modelView?.viewBanners?.title = nil
                self.modelView?.viewBanners?.url = $0.url
                self.modelView?.viewBanners?.collectionBanners.get(banners: banners, reload: false)
                self.modelView?.viewBanners?.flex.markDirty()
                
            case Type.menu:
                
                //                print("catalogMenu")
                
                catalogMenu.allCategoriesPush = self
                catalogMenu.flex.isIncludedInLayout(true)
                catalogMenu.flex.markDirty()
                
            case Type.sections:
                
                //                print("viewSection")
                
                guard let sections = $0.sections else { return }
                guard sections.count != 0 else { return }
                
                
                self.modelView?.viewSection?.transitionURL = self
                self.modelView?.viewSection?.sectionPush = self
                self.modelView?.viewSection?.title = $0.title
                self.modelView?.viewSection?.url = $0.url
                self.modelView?.viewSection?.collectionSections.get(sections: sections, reload: false)
                self.modelView?.viewSection?.flex.markDirty()
                
                
            case Type.producesSection, Type.produces:
                
                //                print("viewProduct")
                
                guard let products = $0.products else { return }
                guard products.count != 0 else { return }
                
                self.modelView?.viewProducts?["\(index)"]?.transitionURL = self
                self.modelView?.viewProducts?["\(index)"]?.productSectionPush = self
                self.modelView?.viewProducts?["\(index)"]?.title = $0.title
                self.modelView?.viewProducts?["\(index)"]?.url = $0.url
                self.modelView?.viewProducts?["\(index)"]?.productSection.get(products: products, reload: false)
                self.modelView?.viewProducts?["\(index)"]?.flex.markDirty()
                
                index += 1
                
            case Type.publishers:
                
                //                print("viewPublisher")
                
                guard let publishers = $0.publishers else { return }
                guard publishers.count != 0 else { return }
                
                self.modelView?.viewPublisher?.transitionURL = self
                self.modelView?.viewPublisher?.publisherPush = self
                self.modelView?.viewPublisher?.title = $0.title
                self.modelView?.viewPublisher?.url = $0.url
                self.modelView?.viewPublisher?.collectionPublishers.get(publisher: publishers, reload: false)
                self.modelView?.viewPublisher?.flex.markDirty()
                
            default:
                break
            }
        }
        
        layout()
        
    }
    
    
    func parsSearch(products: [Products]) {
        
        scrollView.isHidden = true
        prodSecFill.isHidden = false

        prodSecFill.addItemProdSec = self
        prodSecFill.prodSecFillPush = self
        prodSecFill.refreshProdSec = self
        prodSecFill.products = products
        
        
        
        
        
        
//        prodSecFill.flex.isIncludedInLayout(true)
        
//        rootFlexContainer.flex.addItem(prodSecFill).height(view.frame.height - (self.tabBarController?.tabBar.frame.size.height ?? 0) - 15).grow(1)
 
    }
    
    
    
    func addViewForRootFlex(products: [Products]) {
        
        scrollView.contentInset = UIEdgeInsets(top: 40, left: 0, bottom: 0, right: 0)
        scrollView.scrollIndicatorInsets = UIEdgeInsets(top: 40, left: 0, bottom: 0, right: 0)
        
        products.forEach {
            let productSecV4 = ProductSecV4(frame: CGRect(x: 0, y: 0, width: 0, height: heightProdSecV4))
            productSecV4.tag = indexTag
            productSecV4.products = $0
            
            productSecV4.flex.markDirty()
            rootFlexContainer.flex.addItem(productSecV4).height(heightProdSecV4).marginTop(8)
            indexTag += 1
        }
        
        layout()
    }
}



extension ScrollCatalog: TransitionURL {
    func transitionURL(object: Any?, title: String?) {
        
        if let url = object as? String {
            let webViewVC = WebViewVC()
            webViewVC.url = url
            navigationController?.pushViewController(webViewVC, animated: true)
        } else {
            
            if let search = object as? [String: Any] {
//                let catalogSubsection = CatalogSubsection()
//                catalogSubsection.urlSearch = true
//                catalogSubsection.filters = search["filter"] as! [[String : Any]]
//                catalogSubsection.search = search
//                catalogSubsection.navigationItem.title = title
//                navigationController?.pushViewController(catalogSubsection, animated: true)
                
                let scrollCatalog = ScrollCatalog()
                scrollCatalog.urlSearch = true
                scrollCatalog.filters = search["filter"] as! [[String : Any]]
                scrollCatalog.search = search
                scrollCatalog.navigationItem.title = title
                navigationController?.pushViewController(scrollCatalog, animated: true)
                
                
                
            } else if let idProduct = object as? (String, Int) {
                
                switch idProduct.0 {
                case "info":
                    let productInfoTV = TVProductInfo()//ProductInfoTV()
                    productInfoTV.firstOpen = true
                    productInfoTV.idProduce = idProduct.1
                    productInfoTV.navigationItem.title = title
                    navigationController?.pushViewController(productInfoTV, animated: true)
                case "catalog":
                    print("idProduct", idProduct.1)

//                    let catalogSubsection = CatalogSubsection()
//                    catalogSubsection.id = idProduct.1
//                    catalogSubsection.navigationItem.title = title
//                    navigationController?.pushViewController(catalogSubsection, animated: true)
                    print("test 3")
                    
                    let scrollCatalog = ScrollCatalog()
                    scrollCatalog.id = idProduct.1
                    scrollCatalog.navigationItem.title = title
                    navigationController?.pushViewController(scrollCatalog, animated: true)
                    
                    
                default:
                    break
                }
                
            }
            
        }
        
        
        
    }
}


























